# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
from jdu.core import exceptions
from jdu.api.base import JDUWebAPI


class ReceiveCode(JDUWebAPI):

    def get_code(self, material_id=None, coupon_link=None, material_type=1, plan_id=None, is_pin_gou=0,
                 promotion_type=15, receive_type='cps', ware_url=None, is_smart_graphics=None):

        url = 'https://union.jd.com/api/receivecode/getCode'

        ext_headers = {
            'referer': 'https://union.jd.com/proManager/index?pageNo=1',
        }

        data = {
            'couponLink': coupon_link,
            'materialId': material_id,
            'materialType': material_type,
            'isPinGou': is_pin_gou,
            'planId': plan_id,
            'promotionType': promotion_type,
            'receiveType': receive_type,
            'wareUrl': ware_url,
            'isSmartGraphics': is_smart_graphics,
            # 'requestId': 's_45f85c81ee1a4dadb1678b5e56317737'
        }

        package = {
            'data': data
        }

        def decode_result(result, response, client=None, **kwargs):
            if result.get('code') != 200:
                raise exceptions.JDUClientException(
                    result.get('code'),
                    result.get('message'),
                    client=client,
                    request=response.request,
                    response=response
                )
            result = result.get('data', {})
            return result

        return self.post(url, package, ext_headers=ext_headers, result_processor=decode_result)



# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)

from jdu.client.base import BaseClient
from jdu.api.open.category import Category
from jdu.api.open.goods import Goods
from jdu.api.open.order import Order
from jdu.api.open.promotion import Promotion

# todo child_union_id

class JDUOpenApiClient(BaseClient):
    goods = Goods()
    category = Category()
    promotion = Promotion()
    order = Order()

    def __init__(self, app_key, app_secret, site_id=None, sub_union_id=None, version='1.0', timeout=None, auto_retry=True, *args, **kwargs):
        super(JDUOpenApiClient, self).__init__(timeout, auto_retry, *args, **kwargs)
        self.app_key = app_key
        self.app_secret = app_secret
        self.version = version
        self.site_id = site_id
        self.sub_union_id = sub_union_id

    def _request(self, method, uri, **kwargs):

        return super(JDUOpenApiClient, self)._request(method, uri, **kwargs)





# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
import uuid
from jdu.core import exceptions
from jdu.api.base import JDUWebAPI


class Goods(JDUWebAPI):
    def search(self, key=None, page_no=1, page_size=60, category_id=None, cat2_id=None, cat3_id=None, bonus_ids=None,
               delivery_type=0, from_commission_ratio=None, to_commission_ratio=None, from_price=None, to_price=None,
               has_coupon=0, is_hot=None, is_pin_gou=0, is_zy=0, is_care=0, lock=0, orientation_flag=0, sort=None,
               sort_name=None, search_type='st1', keyword_type='kt1'):

        url = 'https://union.jd.com/api/goods/search'

        ext_headers = {
            'referer': 'https://union.jd.com/proManager/index',
        }

        search_uuid = str(uuid.uuid4()).replace('-', '')

        search_data = {
            'key': key,
            'bonusIds': bonus_ids,
            'categoryId': category_id,
            'cat2Id': cat2_id,
            'cat3Id': cat3_id,
            'deliveryType': delivery_type,
            'fromCommissionRatio': from_commission_ratio,
            'toCommissionRatio': to_commission_ratio,
            'fromPrice': from_price,
            'toPrice': to_price,
            'hasCoupon': has_coupon,
            'isHot': is_hot,
            'isPinGou': is_pin_gou,
            'isZY': is_zy,
            'isCare': is_care,
            'lock': lock,
            'orientationFlag': orientation_flag,
            'sort': sort,
            'sortName': sort_name,
            'searchType': search_type,
            'keywordType': keyword_type,
        }

        package = {
            'pageNo': page_no,
            'pageSize': page_size,
            'searchUUID': search_uuid,
            'data': search_data
        }


        return self.post(url, package, ext_headers=ext_headers)



# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
from jdu.core.exceptions import ParamTypeError
from jdu.api.base import JDUOpenAPI


class Goods(JDUOpenAPI):

    def jingfen_query(self, elite_id, page_index=1, page_size=20, sort_name=None, sort='desc', pid=None,
                      fields=None):
        """
        京粉精选商品查询接口
        文档: https://union.jd.com/openplatform/api/10417
        :param elite_id: 频道id：1-好券商品,2-超级大卖场,10-9.9专区,22-热销爆品,23-为你推荐,24-数码家电,25-超市,26-母婴玩具,27-家具日用,28-美妆穿搭,29-医药保健,30-图书文具,31-今日必推,32-品牌好货,33-秒杀商品,34-拼购商品,109-新品首发,110-自营,125-首购商品,129-高佣榜单,130-视频商品
        :param page_index: 页码
        :param page_size: 每页数量，默认20，上限50
        :param sort_name: 排序字段(price：单价, commissionShare：佣金比例, commission：佣金， inOrderCount30DaysSku：sku维度30天引单量，comments：评论数，goodComments：好评数)
        :param sort: asc,desc升降序,默认降序
        :param pid: 联盟id_应用id_推广位id，三段式 eg: 618_618_618
        :param fields: 支持出参数据筛选，逗号','分隔，目前可用：videoInfo
        :return:
        """

        api_method = 'jd.union.open.goods.jingfen.query'

        goods_req = {
            'eliteId': elite_id,
            'pageIndex': page_index,
            'pageSize': page_size,
            'sortName': sort_name,
            'sort': sort,
            'pid': pid,
            'fields': fields,
        }

        params = {
            'goodsReq': goods_req
        }

        return self.get(api_method, params)

    def query(self, cid1=None, cid2=None, cid3=None, page_index=1, page_size=20, sku_ids=None, keyword=None,
              price_from=None, price_to=None, commission_share_start=None, commission_share_end=None, owner=None,
              sort_name=None, sort='desc', is_coupon=None, is_pg=None, pingou_price_start=None, pingou_price_end=None,
              is_hot=None, brand_code=None, shop_id=None, has_content=None, has_best_coupon=None, pid=None, fields=None,
              forbid_types=None, jx_flags=None, shop_level_from=None):
        """
        关键词商品查询接口【申请】
        文档: https://union.jd.com/openplatform/api/10420
        :param cid1:
        :param cid2:
        :param cid3:
        :param page_index:
        :param page_size:
        :param sku_ids:
        :param keyword:
        :param price_from:
        :param price_to:
        :param commission_share_start:
        :param commission_share_end:
        :param owner:
        :param sort_name:
        :param sort:
        :param is_coupon:
        :param is_pg:
        :param pingou_price_start:
        :param pingou_price_end:
        :param is_hot:
        :param brand_code:
        :param shop_id:
        :param has_content:
        :param has_best_coupon:
        :param pid:
        :param fields:
        :param forbid_types:
        :param jx_flags:
        :param shop_level_from:
        :return:
        """

        api_method = 'jd.union.open.goods.query'

        if not sku_ids:
            sku_ids = []

        if not jx_flags:
            jx_flags = []

        if not isinstance(sku_ids, (list, tuple)):
            raise ParamTypeError('sku_ids', 'list')

        if not isinstance(jx_flags, (list, tuple)):
            raise ParamTypeError('jx_flags', 'list')

        goods_req_dto = {
            'cid1': cid1,
            'cid2': cid2,
            'cid3': cid3,
            'pageIndex': page_index,
            'pageSize': page_size,
            'skuIds': sku_ids,
            'keyword': keyword,
            'pricefrom': price_from,
            'priceto': price_to,
            'commissionShareStart': commission_share_start,
            'commissionShareEnd': commission_share_end,
            'owner': owner,
            'sortName': sort_name,
            'sort': sort,
            'isCoupon': is_coupon,
            'isPG': is_pg,
            'pingouPriceStart': pingou_price_start,
            'pingouPriceEnd': pingou_price_end,
            'isHot': is_hot,
            'brandCode': brand_code,
            'shopId': shop_id,
            'hasContent': has_content,
            'hasBestCoupon': has_best_coupon,
            'pid': pid,
            'fields': fields,
            'forbidTypes': forbid_types,
            'jxFlags': jx_flags,
            'shopLevelFrom': shop_level_from,

        }

        params = {
            'goodsReqDTO': goods_req_dto
        }

        return self.get(api_method, params)

    def promotiongoodsinfo_query(self, sku_ids):
        """
        根据skuid查询商品信息接口
        文档: https://union.jd.com/openplatform/api/10422

        :param sku_ids: 京东skuID串，逗号分割，最多100个，开发示例如param_json={'skuIds':'5225346,7275691'}（非常重要 请大家关注：如果输入的sk串中某个skuID的商品不在推广中[就是没有佣金]，返回结果中不会包含这个商品的信息）
        :return:
        """
        api_method = 'jd.union.open.goods.promotiongoodsinfo.query'

        params = {
            'skuIds': sku_ids
        }

        return self.get(api_method, params)




# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
from jdu.api.base import JDUOpenAPI


class Promotion(JDUOpenAPI):

    def common_get(self, material_id, site_id=None, position_id=None, sub_union_id=None,
                   ext1=None, pid=None, coupon_url=None,
                   gift_coupon_key=None):
        """
        网站/APP获取推广链接接口
        文档: https://union.jd.com/openplatform/api/10421

        :param material_id: 推广物料 eg: https://item.jd.com/23484023378.html
        :param site_id: 站点ID是指在联盟后台的推广管理中的网站Id、APPID（1、通用转链接口禁止使用社交媒体id入参；2、订单来源，即投放链接的网址或应用必须与传入的网站ID/AppID备案一致，否则订单会判“无效-来源与备案网址不符”）
        :param position_id: 推广位id
        :param sub_union_id: 子联盟ID（需申请，申请方法请见https://union.jd.com/helpcenter/13246-13247-46301），该字段为自定义参数，建议传入字母数字和下划线的格式
        :param ext1: 系统扩展参数，无需传入
        :param pid: 联盟子站长身份标识，格式：子站长ID_子站长网站ID_子站长推广位ID eg: 618_618_6018
        :param coupon_url: 优惠券领取链接，在使用优惠券、商品二合一功能时入参，且materialId须为商品详情页链接 eg: http://coupon.jd.com/ilink/get/get_coupon.action?XXXXXXX
        :param gift_coupon_key: 礼金批次号 eg: xxx_coupon_key
        :return:
        """

        api_method = 'jd.union.open.promotion.common.get'

        if not site_id:
            site_id = self._client.site_id

        if not sub_union_id:
            sub_union_id = self._client.sub_union_id

        promotion_code_req = {
            'materialId': material_id,
            'siteId': site_id,
            'positionId': position_id,
            'subUnionId': sub_union_id,
            'ext1': ext1,
            'pid': pid,
            'couponUrl': coupon_url,
            'giftCouponKey': gift_coupon_key,
        }

        params = {
            'promotionCodeReq': promotion_code_req
        }

        return self.get(api_method, params)



# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
import json
import uuid

import arrow

from jdu.core import exceptions
from jdu.api.base import JDUWebAPI


class M(JDUWebAPI):

    def get_content_type(self):
        return None

    def list_order_sku(self, order_ids=None, start_time=None, end_time=None, order_status=0, opt_type=1, page_no=1,
                       page_size=20, login_type=3):

        url = 'https://api.m.jd.com/'

        ext_headers = {
            'authority': 'api.m.jd.com',
            'pragma': 'no-cache',
            'cache-control': 'no-cache',
            'referer': 'https://union.jd.com/order',
            'origin': 'https://union.jd.com',
            'sec-fetch-site': 'same-site',
        }

        if not order_ids:
            order_ids = []

        if not isinstance(order_ids, (list)):
            order_ids = [order_ids]

        now = arrow.utcnow().to('+08:00')

        if not end_time:
            end_time = now.replace(hour=23, minute=59, second=59).format('YYYY-MM-DD HH:mm:ss')

        if not start_time:
            start_time = now.shift(days=-7).replace(hour=0, minute=0, second=0).format('YYYY-MM-DD HH:mm:ss')

        body_params = {
            'startTime': start_time,
            'endTime': end_time,
            'orderStatus': order_status,
            'optType': opt_type,
        }

        if order_ids:
            body_params['orderIds'] = order_ids

        body = {
            'funName': 'listOrderSku',
            'param': body_params,
            'page': {
                'pageNo': page_no,
                'pageSize': page_size,
            }
        }
        params = {
            'appid': 'u',
            'body': json.dumps(body),
            'functionId': 'listOrderSku',
            'loginType': login_type,
        }

        return self.get(url, params=params, ext_headers=ext_headers)



# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
from jdu.api.base import JDUOpenAPI


class Category(JDUOpenAPI):

    def goods_get(self, parent_id, grade):
        """
        商品类目查询接口
        文档: https://union.jd.com/openplatform/api/10434

        :param parent_id: 父类目id(一级父类目为0)
        :param grade: 类目级别(类目级别 0，1，2 代表一、二、三级类目)
        :return:
        """
        api_method = 'jd.union.open.category.goods.get'

        req = {
            'parentId': parent_id,
            'grade': grade
        }

        params = {
            'req': req
        }

        return self.get(api_method, params)



# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
from jdu.core import exceptions
from jdu.api.web.m import M
from jdu.api.web.receive_code import ReceiveCode
from jdu.api.web.goods import Goods
from jdu.client.base import BaseClient, logger

class JDUWebBaseClient(BaseClient):
    def __init__(self, cookie, timeout=None, auto_retry=True, *args, **kwargs):
        super(JDUWebBaseClient, self).__init__(timeout, auto_retry, *args, **kwargs)
        self._cookie = cookie

    def _request(self, method, uri, **kwargs):
        return super(JDUWebBaseClient, self)._request(method, uri, **kwargs)

    def get_cookie(self):
        return self._cookie

class JDUWebUnionClient(JDUWebBaseClient):
    """
    针对 https://union.jd.com/api
    """

    goods = Goods()
    receive_code = ReceiveCode()

    def _top_result_processor(self, res, result, **kwargs):
        if 'code' in result and result['code'] != 200:
            code = result.get('code')
            message = result.get('message', '')
            logger.error("\n【请求地址】: %s\n【请求方法】: %s\n【请求参数】：%s \n%s\n【错误信息】：%s",
                         res.url, res.request.method, kwargs.get('params', ''), kwargs.get('data', ''), result)

            if code == 80001 or code == 402:
                raise exceptions.JDUClientAccessException(
                    code,
                    message,
                    client=self,
                    request=res.request,
                    response=res
                )
            else:
                raise exceptions.JDUClientException(
                    code,
                    message,
                    client=self,
                    request=res.request,
                    response=res
                )
        else:
            top_result = result.get('data')
        return top_result


class JDUWebApiClient(JDUWebBaseClient):
    """
    针对于 'https://api.m.jd.com/'
    """
    m = M()

    def _top_result_processor(self, res, result, **kwargs):
        if 'code' in result and result['code'] != 200:
            code = result.get('code')
            message = result.get('message', '')
            logger.error("\n【请求地址】: %s\n【请求方法】: %s\n【请求参数】：%s \n%s\n【错误信息】：%s",
                         res.url, res.request.method, kwargs.get('params', ''), kwargs.get('data', ''), result)

            if code == 80001 or code == 402:
                raise exceptions.JDUClientAccessException(
                    code,
                    message,
                    client=self,
                    request=res.request,
                    response=res
                )
            else:
                raise exceptions.JDUClientException(
                    code,
                    message,
                    client=self,
                    request=res.request,
                    response=res
                )
        else:
            top_result = result
        return top_result


# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
from setuptools import setup

setup(
    name='jdu',
    version='0.0.1b1',
    packages=['jdu',
              'jdu.api',
              'jdu.api.open',
              'jdu.api.web',
              'jdu.client',
              'jdu.core'
              ],
    url='',
    license='',
    author='joshzda',
    author_email='joshzda@gmail.com',
    description='JingDong Union SDK',
    setup_requires=['wheel'],
    install_requires=[
        "appnope==0.1.0",
        "arrow==0.15.6",
        "backcall==0.1.0",
        "certifi==2020.4.5.1",
        "chardet==3.0.4",
        "decorator==4.4.2",
        "idna==2.9",
        "ipython==7.14.0",
        "ipython-genutils==0.2.0",
        "jedi==0.17.0",
        "parso==0.7.0",
        "pexpect==4.8.0",
        "pickleshare==0.7.5",
        "prompt-toolkit==3.0.5",
        "ptyprocess==0.6.0",
        "Pygments==2.6.1",
        "python-dateutil==2.8.1",
        "requests==2.23.0",
        "six==1.14.0",
        "traitlets==4.3.3",
        "urllib3==1.25.9",
        "wcwidth==0.1.9",
    ],
    python_requires='>=3.6',
)



# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
import json

from jdu.api.base import JDUOpenAPI


class Order(JDUOpenAPI):

    def query(self, query_type, query_time, page_no=1, page_size=20, child_union_id=None, key=None):
        """
        订单查询接口
        文档: https://union.jd.com/openplatform/api/10419
        :param page_no: 页码，返回第几页结果
        :param page_size: 每页包含条数，上限为500
        :param query_type: 订单时间查询类型(1：下单时间，2：完成时间，3：更新时间)
        :param query_time: 查询时间，建议使用分钟级查询，格式：yyyyMMddHH、yyyyMMddHHmm或yyyyMMddHHmmss，如201811031212 的查询范围从12:12:00--12:12:59
        :param child_union_id: 子站长ID（需要联系运营开通PID账户权限才能拿到数据），childUnionId和key不能同时传入
        :param key: 其他推客的授权key，查询工具商订单需要填写此项，childUnionid和key不能同时传入
        :return:
        """


        api_method = 'jd.union.open.order.query'

        order_req = {
            'pageNo': page_no,
            'pageSize': page_size,
            'type': query_type,
            'time': query_time,
            'childUnionId': child_union_id,
            'key': key,
        }

        params = {
            'orderReq': order_req
        }

        def process_result(result, response, client=None, **kwargs):
            if result.get('jd_union_open_order_query_response'):
                result = result.get('jd_union_open_order_query_response')
                _result = result.get('result')
                if _result:
                    result = json.loads(_result)
            return result

        return self.get(api_method, params, result_processor=process_result)

    def row_query(self, query_type, start_time, end_time, page_index=1, page_size=20, child_union_id=None, key=None, fields=None):
        """
        订单行查询接口
        文档: https://union.jd.com/openplatform/api/12707
        :param query_type: 订单时间查询类型(1：下单时间，2：完成时间，3：更新时间)
        :param start_time: 开始时间 格式yyyy-MM-dd HH:mm:ss，与endTime间隔不超过1小时
        :param end_time: 结束时间 格式yyyy-MM-dd HH:mm:ss，与startTime间隔不超过1小时
        :param page_index: 页码
        :param page_size: 每页包含条数，上限为500
        :param child_union_id: 子站长ID（需要联系运营开通PID账户权限才能拿到数据），childUnionId和key不能同时传入
        :param key: 其他推客的授权key，查询工具商订单需要填写此项，childUnionid和key不能同时传入
        :param fields: 出参数据筛选，多项逗号分隔
        :return:
        """
        api_method = 'jd.union.open.order.row.query'

        order_req = {
            'pageIndex': page_index,
            'pageSize': page_size,
            'type': query_type,
            'startTime': start_time,
            'endTime': end_time,
            'childUnionId': child_union_id,
            'key': key,
            'fields': fields,
        }

        params = {
            'orderReq': order_req
        }

        def process_result(result, response, client=None, **kwargs):
            if result.get('jd_union_open_order_row_query_response'):
                result = result.get('jd_union_open_order_row_query_response')
                _result = result.get('result')
                if _result:
                    result = json.loads(_result)
            return result

        return self.get(api_method, params, result_processor=process_result)


# -*- coding: utf-8 -*-
# Authored by: Josh (joshzda@gmail.com)
import hashlib
import json

import arrow


class JDUBaseAPI(object):

    API_BASE_URL = None

    def __init__(self, client=None):
        self._client = client

    def _get(self, url, **kwargs):
        if self.API_BASE_URL:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.get(url, **kwargs)

    def _post(self, url, **kwargs):
        if self.API_BASE_URL:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.post(url, **kwargs)

class JDUOpenAPI(JDUBaseAPI):

    API_BASE_URL = 'https://router.jd.com/api'

    def __init__(self, client=None):
        super(JDUOpenAPI, self).__init__(client)

    def _get_url(self):
        return self.API_BASE_URL

    def _get(self, url, params=None, **kwargs):
        kwargs['params'] = params
        return super(JDUOpenAPI, self)._get(url, **kwargs)

    def get(self, api_method, params, **kwargs):
        _params = self.package(api_method, params)
        return self._get(self._get_url(), _params, **kwargs)

    def _get_access_token(self):
        return None

    def sign_params(self, params):
        params.pop('sign')
        params.pop('sign_method')
        app_secret = self._client.app_secret
        _sign_list = []
        for key in sorted(params.keys()):
            s = '%s%s' % (key, params[key])
            _sign_list.append(s)
        _sign = ''.join(_sign_list)
        _sign = '%s%s%s' % (app_secret, _sign, app_secret)
        _sign = hashlib.md5(_sign.encode("utf-8")).hexdigest().upper()
        return _sign

    def package(self, api_method, params):
        param_json = json.dumps(params)
        timestamp = arrow.utcnow().to('+08:00').format('YYYY-MM-DD HH:mm:ss')

        access_token = self._get_access_token()

        system_params = {
            'method': api_method,
            'app_key': self._client.app_key,
            'format': 'json',
            'v': self._client.version,
            'param_json': param_json,
            'timestamp': timestamp,
            'sign': '',
            'sign_method': 'md5'
        }

        if access_token:
            system_params['access_token'] = access_token

        sign = self.sign_params(system_params)
        system_params['sign'] = sign
        return system_params


class JDUWebAPI(JDUBaseAPI):

    API_BASE_URL = 'https://union.jd.com/api/'

    def __init__(self, client=None):
        super(JDUWebAPI, self).__init__(client)

    def get_content_type(self):
        return u'application/json;charset=UTF-8'

    def get_header(self, ext_headers=None):
        _headers = {
            'accept': 'application/json, text/plain, */*',
            'authority': 'union.jd.com',
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-mode': 'cors',
            'sec-fetch-dest': 'empty',
            'accept-language': 'zh-CN,zh;q=0.9,en;q=0.8',
            'cookie': self._client.get_cookie()
        }

        content_type = self.get_content_type()
        if content_type:
            _headers['Content-Type'] = content_type

        if isinstance(ext_headers, dict):
            _headers.update(ext_headers)

        return _headers

    def _get(self, url, params=None, ext_headers=None, **kwargs):
        kwargs['params'] = params
        kwargs['headers'] = self.get_header(ext_headers)
        return super(JDUWebAPI, self)._get(url, **kwargs)

    def _post(self, url, params=None, data=None, ext_headers=None, **kwargs):
        kwargs['params'] = params
        kwargs['data'] = data
        kwargs['headers'] = self.get_header(ext_headers)
        return super(JDUWebAPI, self)._get(url, **kwargs)

    def get(self, url, params=None, ext_headers=None, **kwargs):
        return self._get(url, params, ext_headers, **kwargs)

    def post(self, url, data=None, params=None, ext_headers=None, **kwargs):
        return self._post(url, params, data, ext_headers, **kwargs)